#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "DJR_Audiosystem.h"

static int m_iDSP_FileHandle;
static int m_iFormat;
static int m_iFrequency;
static int m_iBitsPerSample;
static int m_iChannels;
static int m_iFragmentNumAndSize;

/************************************************/
void AudiosystemInit( void )
{
  printf( "\ninitializing the audio system:" );

  if( ( m_iDSP_FileHandle = open( "/dev/dsp", O_WRONLY ) ) < 0 )
  {
    printf( "\nFAILED TO OPEN /dev/dsp!!!!!" );
    exit( 1 );
  }

  m_iFragmentNumAndSize = LOG2_FRAG_SIZE | ( NUM_FRAGS << 16 );
  ioctl( m_iDSP_FileHandle, SNDCTL_DSP_SETFRAGMENT, &m_iFragmentNumAndSize );
  printf( "\n   bytes per fragment = %d", (int)powf( 2, m_iFragmentNumAndSize & 0x0000FFFF ) );
  printf( "\n   number of fragments = %d", m_iFragmentNumAndSize >> 16 );

  m_iFormat = AFMT_S16_LE;
  ioctl( m_iDSP_FileHandle, SNDCTL_DSP_SETFMT, &m_iFormat );
  printf( "\n   OSS format enum = %d", m_iFormat );

  m_iFrequency = 48000;
  ioctl( m_iDSP_FileHandle, SNDCTL_DSP_SPEED, &m_iFrequency );
  printf( "\n   frequency = %d", m_iFrequency );

  m_iBitsPerSample = 16;
  ioctl( m_iDSP_FileHandle, SNDCTL_DSP_SAMPLESIZE, &m_iBitsPerSample );
  printf( "\n   bits per sample = %d", m_iBitsPerSample );

  m_iChannels = 2;
  ioctl( m_iDSP_FileHandle, SNDCTL_DSP_CHANNELS, &m_iChannels );
  printf( "\n   channels = %d", m_iChannels );

  // for debugging: see what the device supports
  int caps;
  ioctl( m_iDSP_FileHandle, SNDCTL_DSP_GETCAPS, &caps );
  printf( "\n\nOSS DRIVER AND DEVICE INFO:" );
  printf( "\n   DSP_CAP_REVISION: %d", caps & 0x000000FF );
  printf( "\n   full duplex: %s", caps & DSP_CAP_DUPLEX? "yes":"no" );
  printf( "\n   precise pointers with SNDCTL_DSP_GETxPTR: %s", caps & DSP_CAP_REALTIME? "yes":"no" );
  printf( "\n   local storage for playback: %s", caps & DSP_CAP_BATCH? "yes":"no" );
  printf( "\n   programmable DSP chip: %s", caps & DSP_CAP_COPROC? "yes":"no" );
  printf( "\n   playback via triggers: %s", caps & DSP_CAP_TRIGGER? "yes":"no" );
  printf( "\n   direct access to hardware playback buffer: %s", caps & DSP_CAP_MMAP? "yes":"no" );

  fflush( NULL );
}

/************************************************/
void AudiosystemKill( void )
{
  close( m_iDSP_FileHandle );
}

/************************************************/
int OSSDeviceMaxBytes( void )
{
  audio_buf_info info;
  //ioctl( m_iDSP_FileHandle, SNDCTL_DSP_GETISPACE, &info );
  ioctl(m_iDSP_FileHandle, SNDCTL_DSP_GETOSPACE, &info );

  //printf( "\nwe can write %d bytes (%d fragments) without blocking!", info.bytes, info.fragments );

  return info.bytes;
}

/************************************************/
int OSSDeviceNumFrags( void ) 
{ 
  return( m_iFragmentNumAndSize >> 16 ); 
}

/************************************************/
Bool WriteBuffToDSP( signed short *pBuff )
{
  // only write one fragment at a time
  if( write( m_iDSP_FileHandle, pBuff, FRAG_SIZE ) < FRAG_SIZE )
  {
    printf( "\nUH OH!!!  failed to write 1024 bytes to /dev/dsp" );
    return FALSE;
  }
  return TRUE;
}

/**************************************************/
Bool WriteSampleToFragment( signed short *pSample )
{
  // only write one fragment at a time
  if( write( m_iDSP_FileHandle, pSample, 4 ) < 4 )
  {
    //printf( "\nUH OH!!!  failed to write a 4 byte sample to /dev/dsp" );
    return FALSE;
  }
  return TRUE;
}

/************************************************/
int GetFragSize( void )
{
  // TODO: replace this with a table or approximation
  return( (int)powf( 2, ( m_iFragmentNumAndSize & 0x0000FFFF ) ) );
}

/************************************************/
void DBG_PlayTestNoise_BLOCKING( int iFrags, signed short *pExternalBuff )
{
  signed short pMixedFragmentBuff[FRAG_SIZE/2] __attribute__((aligned(16)));
  unsigned int times, bytes;
  char *pChar;

  for( times=0; times<iFrags; times++ )
  {
    // fill the local buffer
    if( pExternalBuff )
    {
      memcpy( pMixedFragmentBuff, pExternalBuff, 1024 );
    }
    else
    {
      for( bytes=0, pChar=(char*)pMixedFragmentBuff; bytes<1024; bytes++ )
      {
	*pChar = random() % 256;
        pChar++;
      }
    }

    while( OSSDeviceMaxBytes() < 1024 );
    WriteBuffToDSP( pMixedFragmentBuff );
  }  
}
