#ifndef _DJR_MIXER_H_
#define _DJR_MIXER_H_

#include "UsefulMacros.h"

#define MAX_PLAYBACK_SPEED ( 4.f )

typedef Bool (*MixerEventCallbackFunc)(int, void *);

// types of mixer events that can have callbacks
// for simplicity, each event can have ONE callback
typedef enum 
{
  MIXER_EVENT_LOOP_NEW,
  MIXER_EVENT_LOOP_HALF,
  MIXER_EVENT_LOOP_QUARTER,
  MIXER_EVENT_LOOP_EIGHTH,
  MIXER_EVENT_LOOP_PAUSE,
  MIXER_EVENT_NEW_RECORD,
  MIXER_EVENT_NEW_TRACK,
  MIXER_EVENT_ALL,
  MIXER_EVENT_MAX = MIXER_EVENT_ALL
}MixerEvent;

typedef enum
{
  SCRATCH_MODE_HALF,
  SCRATCH_MODE_FULL,
  SCRATCH_MODE_MAX_MODES
}MixerScratchMode;

// keep track of each turntable
typedef struct
{
  float fNumSamples; // file size / 2 channels / 2 bytes per sample
  float fSpeed;  // 1=normal, 2=double, 0.5=half, -1=normal backwards
  float fCurPos; // current sample.  x2 (stereo) for the real position
  float fInitialBPM; // assumes loop is 2 measures (8 beats) in 4/4 time
  Bool bLoop; // loop or one-shot
  Bool bLoaded; // do we have a valid sample loaded
  Bool bPlaying; // playing or paused
}SampleState;

// when we switch to the next track, we stream.
typedef struct
{
  Bool bStreaming;          // currently streaming a track? 
  int iFileHandle;          // file handle to read from
  unsigned long iFileSize;  // size of the file we're reading (in bytes)
  unsigned long iBytesRead; // how many bytes read so far
  unsigned int iTrackNum;   // track number to stream
}TrackStreamState;

void MixerInit( void );
void MixerKill( void );
Bool MixerLoadSample( char *pFileName, char iTable, 
                      float fSpeed, Bool bLoop, Bool bReadData );
void MixerUnloadSample( char iTurntable );
Bool MixerTogglePause( char iTurntable );
void MixerProcessScratching( void );
Bool MixerProcess( Bool bReset, Bool bTimerExpired );
void MixerRenderDebugText( void );
float MixerGetRecordSpeed( char iTurntable );
void MixerEventAddCallback( MixerEvent eEvent, 
                            MixerEventCallbackFunc pCallbackFunc );
void MixerEventRemoveCallback( MixerEvent eEvent, 
                               MixerEventCallbackFunc pCallbackFunc );
 
#endif
