#include "DJR_Visuals.h"
#include "DJR_SimpleTexMngr.h"
#include "DJR_Mixer.h"
#include <sps2types.h>
#include "sps2util_Godai.h"

enum
{
  TEX_SLOT_MIXER,
  TEX_SLOT_REC_1,
  TEX_SLOT_REC_2,
  TEX_SLOT_TURNTABLE,
  TEX_SLOT_VIS_MAX,
};

typedef enum
{
  EFFECT_NONE,
  EFFECT_HYPER,
  EFFECT_TRIPPY,
  EFFECT_MAX
}eRecordEffect;

// constants
#define MIXER_HEIGHT ( 3100 )
#define MIXER_WIDTH ( 1800 * 2 )
#define RECORD_WIDTH ( ( int )( MIXER_HEIGHT * 0.75f ) )
#define RECORD_HEIGHT RECORD_WIDTH


// statics
static unsigned char _iBGColorCurrent[ 3 ] = { 0, 64, 255 };
static unsigned char _iBGColorTarget[ 3 ] = { 0, 64, 255 };
static int _iMixerPos[ 2 ] = { 0, 1550 };
static int _iRecordPos[ 2 ][ 2 ] = { { -3500, 1550 }, 
                                     { 3500, 1550 } };
static float _fRecordRot[ 2 ] = { 0.f, 0.f };

// prototypes
void _GetRecordRotation( int iRec, int *ulx, int *uly, int *llx, int *lly, 
                         int *lrx, int *lry, int *urx, int *ury, 
                         eRecordEffect Effect );

Bool _LoopQuarterUpdateBkgnd( int iTable, void *pData );

/*******************************************************************/
void InitVisuals( void )
{
  UploadTexToGS( "textures/mixer_tex.raw", TEX_SLOT_MIXER, 
                 TEX_PRI_NEVER_DUMP, "mixer" );

  UploadTexToGS( "textures/record_tex1.raw", TEX_SLOT_REC_1, 
                 TEX_PRI_NEVER_DUMP, "rec1" );

  char iColorCnt;
  for( iColorCnt = 0; iColorCnt < 3; iColorCnt++ )
  {
    _iBGColorTarget[ iColorCnt ] = random( ) % 256;
    _iBGColorCurrent[ iColorCnt ] = random( ) % 256;
  }

  MixerEventAddCallback( MIXER_EVENT_LOOP_QUARTER,
                         _LoopQuarterUpdateBkgnd );
}

/*******************************************************************/
void RenderRecords( void )
{
  sps2int32 ulx, uly, llx, lly, lrx, lry, urx, ury;

  _GetRecordRotation( 0, &ulx, &uly, &llx, &lly, 
                      &lrx, &lry, &urx, &ury, EFFECT_NONE );

  DrawTexturedSprite( ulx, uly, llx, lly, lrx, lry, urx, ury, 
                      TEX_W, TEX_H, TEX_SLOT_REC_1 );

  _GetRecordRotation( 1, &ulx, &uly, &llx, &lly, 
                      &lrx, &lry, &urx, &ury, EFFECT_NONE );

  DrawTexturedSprite( ulx, uly, llx, lly, lrx, lry, urx, ury, 
                      TEX_W, TEX_H, TEX_SLOT_REC_1 );
}

/******************************************************************/
void RenderMixer( void )
{
  DrawTexturedSprite( _iMixerPos[ 0 ] - 0.5f * MIXER_WIDTH, 
                      _iMixerPos[ 1 ] - 0.5f * MIXER_HEIGHT,
                      _iMixerPos[ 0 ] - 0.5f * MIXER_WIDTH, 
                      _iMixerPos[ 1 ] + 0.5f * MIXER_HEIGHT, 
                      _iMixerPos[ 0 ] + 0.5f * MIXER_WIDTH, 
                      _iMixerPos[ 1 ] + 0.5f * MIXER_HEIGHT,
                      _iMixerPos[ 0 ] + 0.5f * MIXER_WIDTH, 
                      _iMixerPos[ 1 ] - 0.5f * MIXER_HEIGHT,  
                      TEX_W, TEX_H, 
                      TEX_SLOT_MIXER );
  MixerRenderDebugText( );
}

/******************************************************************/
void RenderBackgroundVisuals( void )
{
  char iColorCnt;

  sps2UScreenClear( _iBGColorCurrent[ 0 ],
                    _iBGColorCurrent[ 1 ],
                    _iBGColorCurrent[ 2 ] );
  
  // interp to the current target 
  for( iColorCnt = 0; iColorCnt < 3; iColorCnt++ )
    _iBGColorCurrent[ iColorCnt ] +=
      _iBGColorCurrent[iColorCnt] == _iBGColorTarget[iColorCnt] ? 0 :
        (_iBGColorTarget[iColorCnt] > _iBGColorCurrent[iColorCnt] ? 1 : 
           -1 );
}

/********************************************************************/
void _GetRecordRotation( int iRec, int *ulx, int *uly, int *llx, int *lly, 
                         int *lrx, int *lry, int *urx, int *ury,
                         eRecordEffect Effect )
{
  int iHalfWidth = ( int )( RECORD_WIDTH * 0.5f );
  int iHalfHeight = ( int )( RECORD_HEIGHT * 0.5f );

  int tmpx;

  // untranslated record
  *ulx = *llx = -iHalfWidth;
  *uly = *ury = -iHalfHeight;
  *urx = *lrx = iHalfWidth;
  *lly = *lry = iHalfHeight;

  // rotate
  float fRad = _fRecordRot[ iRec ] * PIOVER180;
  float fSin = sinf( fRad );
  float fCos = cosf( fRad );

  tmpx = ( int )( fCos * ( *ulx ) - fSin * ( *uly ) );
  *uly = ( int )( fSin * ( *ulx ) + fCos * ( *uly ) );
  *ulx = tmpx;

  tmpx = ( int )( fCos * ( *llx ) - fSin * ( *lly ) );
  *lly = ( int )( fSin * ( *llx ) + fCos * ( *lly ) );
  *llx = tmpx;

  tmpx = ( ( int )( fCos * ( *lrx ) - fSin * ( *lry ) ) );
  *lry = ( ( int )( fSin * ( *lrx ) + fCos * ( *lry ) ) );
  *lrx = tmpx;

  tmpx = ( int )( fCos * ( *urx ) - fSin * ( *ury ) );
  *ury = ( int )( fSin * ( *urx ) + fCos * ( *ury ) );
  *urx = tmpx;

  // translate
  *ulx += _iRecordPos[ iRec ][ 0 ]; 
  *uly += _iRecordPos[ iRec ][ 1 ]; 
  *llx += _iRecordPos[ iRec ][ 0 ]; 
  *lly += _iRecordPos[ iRec ][ 1 ];
  *lrx += _iRecordPos[ iRec ][ 0 ]; 
  *lry += _iRecordPos[ iRec ][ 1 ]; 
  *urx += _iRecordPos[ iRec ][ 0 ]; 
  *ury += _iRecordPos[ iRec ][ 1 ];

  // update the angle
  _fRecordRot[ iRec ] += 4.f * MixerGetRecordSpeed( iRec );

  if( _fRecordRot[ iRec ] >= 360.f )
      _fRecordRot[ iRec ] = 0.f;
  else if( _fRecordRot[ iRec ] < 0.f )
      _fRecordRot[ iRec ] = 359.f;
}

/*****************************************************************/
Bool _LoopQuarterUpdateBkgnd( int iTable, void *pData )
{
    int iColorCnt;

    //printf( "\n\n_LoopQuarterUpdateBkgnd called for loop 1/4 event" );

    for( iColorCnt = 0; iColorCnt < 3; iColorCnt++ )
    {    
        _iBGColorTarget[ iColorCnt ] = random( ) % 256;
        _iBGColorCurrent[ iColorCnt ] = random( ) % 256;
    }
}
