#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <string.h>
#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include "gsregs.h"

// types
typedef float vertex_t[ 3 ];
typedef float normal_t[ 3 ];
typedef float texvert_t[ 2 ];
typedef int face_t[ 9 ];

// prototypes
void count_elements( FILE *fp,  unsigned int *num_verts, 
                     unsigned int *num_normals, 
                     unsigned int *num_tex_verts, 
                     unsigned int *num_faces );

void create_dma_file( FILE *fp, char *outfile, 
                      int num_verts, int num_normals, 
                      int num_tex_verts, int num_faces );

/**************************************/
int main( int argc, char **argv ) 
{
  unsigned int num_verts = 0;
  unsigned int num_normals = 0;
  unsigned int num_tex_verts = 0;
  unsigned int num_faces = 0;
  FILE *fp;
	
  if( argc != 4 )
  {
    printf( "\n\nusage:\nobj2dma model.obj outfile.dma scale\n\n" );
    exit( 1 );		
  }
	
  if( ( fp = fopen( argv[ 1 ], "r" ) ) == NULL )
  //if( ( fp = fopen( "bad-table.obj", "r" ) ) == NULL )
  {
    printf( "\n\ncant open file %s\n\n", argv[ 1 ] );
    exit( 2 );
  }
	
  int iSPS2Device = sps2Init( );
	
  count_elements( fp, &num_verts, &num_normals, 
                  &num_tex_verts, &num_faces );
	
  create_dma_file( fp, argv[ 2 ], num_verts, num_normals, 
                   num_tex_verts, num_faces );

  fclose( fp );
  return 0;
}

/*********************************/
void count_elements( FILE *fp, unsigned int *num_verts, 
                     unsigned int *num_normals,
                     unsigned int *num_tex_verts, 
                     unsigned int *num_faces )
{
  char line[ 128 ];  // way bigger than we need
	
  *num_verts = 0;
  *num_normals = 0;
  *num_tex_verts = 0;
  *num_faces = 0;
	
  while( !feof( fp ) )
  {
    fgets( line, 128, fp );
		
    if( strncmp( line, "v ", 2 ) == 0 )
      (*num_verts)++;
		
    else if( strncmp( line, "vn", 2 ) == 0 )
      (*num_normals)++;
					
    else if( strncmp( line, "vt", 2 ) == 0 )
      (*num_tex_verts)++;
							
    else if( strncmp( line, "f ", 2 ) == 0 )
      (*num_faces)++;
  }
  fseek( fp, SEEK_SET, 0 );
}

/***********************************/
void create_dma_file( FILE *fp, char *outfile, int num_verts, 
                      int num_normals, int num_tex_verts, 
                      int num_faces )
{
  int fh = -1;
  sps2GIFTag_t gt;
  sps2uint64 regs = 0;
  sps2GIFPackedRegister_t pr;
  int vert_index = 0;
  int norm_index = 0;
  int tex_vert_index = 0;
  vertex_t *vertices;
  normal_t *normals;
  texvert_t *texverts;
  char line[ 128 ];  // way bigger than we need
	
  vertices = ( vertex_t * )malloc( sizeof( vertex_t ) * num_verts );
  normals = ( normal_t * )malloc( sizeof( normal_t ) * num_normals );
  texverts = ( texvert_t * )malloc( sizeof( texvert_t ) * num_tex_verts );
	
  fh = open( outfile, O_CREAT | O_RDWR );
	
  // a face is a triangle, and each triangle has 3 verts 
  // and ST texture verts
  gt.i128 = 0;
  gt.s.NLOOP = num_faces;
  gt.s.EOP = 1;
  gt.s.PRE = 0;
  gt.s.PRIM = 0;
  gt.s.FLG = GIF_FLG_PACKED;
  gt.s.NREG = 0;
  SPS2_SET_GIF_REG(gt,gt.s.NREG++,GIF_REG_ST);   // ST
  SPS2_SET_GIF_REG(gt,gt.s.NREG++,GIF_REG_A_D);  // XYZ2
  SPS2_SET_GIF_REG(gt,gt.s.NREG++,GIF_REG_ST);   // ST
  SPS2_SET_GIF_REG(gt,gt.s.NREG++,GIF_REG_A_D);  // XYZ2
  SPS2_SET_GIF_REG(gt,gt.s.NREG++,GIF_REG_ST);   // ST
  SPS2_SET_GIF_REG(gt,gt.s.NREG++,GIF_REG_A_D);  // XYZ2
	
  // write the GIFTag
  write( fh, &gt, sizeof( sps2GIFTag_t ) );
	
  while( !feof( fp ) )
  {
    fgets( line, 128, fp );
		
    if( strncmp( line, "v ", 2 ) == 0 )
    {
      sscanf( line, "v %f %f %f", &vertices[ vert_index ][ 0 ], 
                                  &vertices[ vert_index ][ 1 ], 
                                  &vertices[ vert_index ][ 2 ] );
      ++vert_index;
    }

    else if( strncmp( line, "vn", 2 ) == 0 )
    {
      sscanf( line, "vn %f %f %f", &normals[ norm_index ][ 0 ], 
                                   &normals[ norm_index ][ 1 ], 
                                   &normals[ norm_index ][ 2 ] );
      ++norm_index;
    }
		
    else if( strncmp( line, "vt", 2 ) == 0 ) 			
    {
      sscanf( line, "vt %f %f", &texverts[ tex_vert_index ][ 0 ], 
                                &texverts[ tex_vert_index ][ 1 ] );
      ++tex_vert_index;
    }
		
    else if( strncmp( line, "f ", 2 ) == 0 )
    {
      int v[ 3 ], t[ 3 ], n[ 3 ];
      sscanf( line, "f %d/%d/%d %d/%d/%d %d/%d/%d", 
                           &v[ 0 ], &t[ 0 ], &n[ 0 ], 
	                        &v[ 1 ], &t[ 1 ], &n[ 1 ], 
                           &v[ 2 ], &t[ 2 ], &n[ 2 ] );

      printf( "\n\ntriangle:" );

      int iVert = 0;
      for( iVert=0; iVert<3; iVert++ )
      {
         // ST
	 pr.ST.i128 = 0;
	 pr.ST.s.S = texverts[ t[ iVert ] - 1 ][ 0 ];
	 pr.ST.s.T = texverts[ t[ iVert ] - 1 ][ 1 ];
	 pr.ST.s.Q = 1;
	 write( fh, &pr, sizeof( sps2GIFPackedRegister_t ) );
         //printf( "\nST = %f, %f", pr.ST.s.S, pr.ST.s.T );
				
         // my apologies for this crappiness.  converting to fixed
         // point and scaling was an afterthought so I quickly hacked
         // it in instead of properly rewriting stuff

         sps2int32 fixed_point[4] __attribute__ ((aligned(16)));
         float reverse[4] __attribute__ ((aligned(16)));
         float aligned_verts[4] __attribute__ ((aligned(16))) = 
	     { 123.45f, -0.45f, 0.45f, 0.55f }; 
         float scalevec[4] __attribute__ ((aligned(16))) = 
	     { 1.f, 1.f, 1.f, 1.f };

         memcpy( &aligned_verts, 
	         vertices[ v[ iVert] - 1 ], 
                 sizeof(float) * 3 );

	 // scale and convert to fixed point
	 asm __volatile__("
             lqc2 vf1, 0(%1)
             lqc2 vf4, 0(%3)
             VMUL.xyz vf1xyz, vf1xyz, vf4
             VFTOI4.xy vf2xy, vf1xy
             VITOF4.xy vf3xy, vf2xy
             sqc2 vf2, 0(%0)
             sqc2 vf3, 0(%2)
             " : : "r" (fixed_point), 
                   "r" (aligned_verts), 
                   "r" (reverse), 
                   "r" (scalevec));
 
         int16_t fixed_x = fixed_point[0], 
                 fixed_y = fixed_point[1];

         printf( "\n\nfixed point: %d %d", fixed_x, 
                                           fixed_y);
         printf( "\nback to float: %f, %f", reverse[ 0 ], reverse[ 1 ] );

	 // XYZ
	 regs = PACK_XYZ2( fixed_x, 
		           fixed_y, 
		           vertices[ v[ iVert ] - 1 ][ 2 ] * scalevec[2] );
	 write( fh, &regs, sizeof( sps2uint64 ) );
	    
	 regs = XYZ2_R;	
	 write( fh, &regs, sizeof( sps2uint64 ) );
         printf( "\nXYZ = %f, %f, %f", vertices[ v[ iVert ] - 1 ][ 0 ], 
                                       vertices[ v[ iVert ] - 1 ][ 1 ],
                                       vertices[ v[ iVert ] - 1 ][ 2 ] );
      }
    }
  }	

  close( fh );
  free( vertices );
  free( normals );
  free( texverts );
}
